@@ -17,7 +17,7 @@
 //!
 //! And create `src/main.rs` along the lines of:
 //!
-//! ```rust,no_run
+//! ```rust,ignore
 //! fn main() {
 //!     sop::cli::main(&MySOPImplementation::default());
 //! }
@@ -45,7 +45,14 @@
 //!
 //! #[cfg(not(feature = "cli"))]
 //! fn main() {}
-//! //! ```
+//! ```
+//!
+//! # Features and limitations
+//!
+//! - The special designator `@FD:` is only available on UNIX-like systems.
+//!
+//! - On Windows, certs and keys provided via the `@ENV:` special
+//!   designator must be ASCII armored and well-formed UTF-8.
 
 use std::{
     io,
@@ -270,7 +277,7 @@ fn real_main(sop: &dyn crate::SOP) -> Result<()> {
                 op = op.not_after(t.into());
             }
             for mut cert in load_files(certs)? {
-                op = op.cert(&mut cert)?;
+                op = op.certs(&mut cert)?;
             }
             let verifications = op.signatures(&mut load_file(signatures)?)?
                 .data(&mut io::stdin())?;
@@ -287,10 +294,10 @@ fn real_main(sop: &dyn crate::SOP) -> Result<()> {
                 op = op.no_armor();
             }
             for mut key in load_files(sign_with)? {
-                op = op.sign_with(&mut key)?;
+                op = op.sign_with_key(&mut key)?;
             }
             for pw in with_password {
-                op = op.with_password(&pw)?;
+                op = op.with_password(pw.into())?;
             }
             for mut cert in load_files(certs)? {
                 op = op.with_cert(&mut cert)?;
@@ -335,7 +342,7 @@ fn real_main(sop: &dyn crate::SOP) -> Result<()> {
             }
 
             for p in with_password {
-                op = op.with_password(&p)?;
+                op = op.with_password(p.into())?;
             }
 
             for mut key in load_files(key)? {
@@ -343,7 +350,7 @@ fn real_main(sop: &dyn crate::SOP) -> Result<()> {
             }
 
             for mut cert in load_files(verify_with)? {
-                op = op.verify_with_cert(&mut cert)?;
+                op = op.verify_with_certs(&mut cert)?;
             }
 
             if let Some(t) = verify_not_before {
@@ -394,7 +401,8 @@ fn is_special_designator<S: AsRef<str>>(file: S) -> bool {
 }
 
 /// Loads the given (special) file.
-fn load_file<S: AsRef<str>>(file: S) -> Result<std::fs::File> {
+fn load_file<S: AsRef<str>>(file: S)
+                            -> Result<Box<dyn io::Read + Send + Sync>> {
     let f = file.as_ref();
 
     if is_special_designator(f) {
@@ -403,11 +411,52 @@ fn load_file<S: AsRef<str>>(file: S) -> Result<std::fs::File> {
                 .context(format!("File {:?} exists", f));
         }
 
+        #[cfg(unix)]
+        {
+            if f.starts_with("@FD:")
+                && f[4..].chars().all(|c| c.is_ascii_digit())
+            {
+                use std::os::unix::io::{RawFd, FromRawFd};
+
+                let fd: RawFd = f[4..].parse()
+                    .map_err(|_| Error::UnsupportedSpecialPrefix)?;
+                let f = unsafe {
+                    std::fs::File::from_raw_fd(fd)
+                };
+                return Ok(Box::new(f));
+            }
+
+            if f.starts_with("@ENV:") {
+                use std::os::unix::ffi::OsStringExt;
+
+                let key = &f[5..];
+                let value = std::env::var_os(key)
+                    .ok_or(Error::UnsupportedSpecialPrefix)?;
+                // Prevent leak to child processes.
+                std::env::remove_var(key);
+                return Ok(Box::new(io::Cursor::new(value.into_vec())));
+            }
+        }
+
+        #[cfg(windows)]
+        {
+            if f.starts_with("@ENV:") {
+                let key = &f[5..];
+                let value = std::env::var(key)
+                    .map_err(|_| Error::UnsupportedSpecialPrefix)?;
+                // Prevent leak to child processes.
+                std::env::remove_var(key);
+                return Ok(Box::new(io::Cursor::new(value.into_bytes())));
+            }
+        }
+
         return Err(anyhow::Error::from(Error::UnsupportedSpecialPrefix));
     }
 
-    std::fs::File::open(f).map_err(|_| Error::MissingInput)
-            .context(format!("Failed to open file {:?}", f))
+    std::fs::File::open(f)
+        .map(|f| -> Box<dyn io::Read + Send + Sync> { Box::new(f) })
+        .map_err(|_| Error::MissingInput)
+        .context(format!("Failed to open file {:?}", f))
 }
 
 /// Creates the given (special) file.
@@ -433,7 +482,8 @@ fn create_file<S: AsRef<str>>(file: S) -> Result<std::fs::File> {
 }
 
 /// Loads the the (special) files.
-fn load_files(files: Vec<String>) -> Result<Vec<std::fs::File>> {
+fn load_files(files: Vec<String>)
+              -> Result<Vec<Box<dyn io::Read + Send + Sync>>> {
     files.iter().map(load_file).collect()
 }
 
