@@ -97,6 +97,12 @@ pub trait Sign<'a> {
     fn key(self: Box<Self>, key: &mut (dyn io::Read + Send + Sync))
            -> Result<Box<dyn Sign<'a> + 'a>>;
 
+    /// Adds the signer keys.
+    ///
+    /// Like [`Sign::key`], but for multiple keys.
+    fn keys(self: Box<Self>, keys: &mut (dyn io::Read + Send + Sync))
+            -> Result<Box<dyn Sign<'a> + 'a>>;
+
     /// Signs data.
     fn data(self: Box<Self>, data: &'a mut (dyn io::Read + Send + Sync))
         -> Result<Box<dyn Ready + 'a>>;
@@ -114,6 +120,12 @@ pub trait Verify<'a> {
     fn cert(self: Box<Self>, cert: &mut (dyn io::Read + Send + Sync))
             -> Result<Box<dyn Verify<'a> + 'a>>;
 
+    /// Adds the verification certs.
+    ///
+    /// Like [`Verify::cert`], but for multiple certs.
+    fn certs(self: Box<Self>, certs: &mut (dyn io::Read + Send + Sync))
+             -> Result<Box<dyn Verify<'a> + 'a>>;
+
     /// Provides the signatures.
     fn signatures(self: Box<Self>,
                   signatures: &'a mut (dyn io::Read + Send + Sync))
@@ -128,7 +140,7 @@ pub trait VerifySignatures {
             -> Result<Vec<Verification>>;
 }
 
-/// Represents a successful signature verification.
+/// A successful signature verification.
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct Verification {
     creation_time: SystemTime,
@@ -213,17 +225,29 @@ pub trait Encrypt<'a> {
     fn mode(self: Box<Self>, mode: EncryptAs) -> Box<dyn Encrypt<'a> + 'a>;
 
     /// Adds the signer key.
-    fn sign_with(self: Box<Self>, key: &mut (dyn io::Read + Send + Sync))
-                 -> Result<Box<dyn Encrypt<'a> + 'a>>;
+    fn sign_with_key(self: Box<Self>, key: &mut (dyn io::Read + Send + Sync))
+                     -> Result<Box<dyn Encrypt<'a> + 'a>>;
+
+    /// Adds the signer keys.
+    ///
+    /// Like [`Encrypt::sign_with_key`], but for multiple keys.
+    fn sign_with_keys(self: Box<Self>, keys: &mut (dyn io::Read + Send + Sync))
+                      -> Result<Box<dyn Encrypt<'a> + 'a>>;
 
     /// Encrypts with the given password.
-    fn with_password(self: Box<Self>, password: &str)
+    fn with_password(self: Box<Self>, password: Password)
                      -> Result<Box<dyn Encrypt<'a> + 'a>>;
 
     /// Encrypts with the given cert.
     fn with_cert(self: Box<Self>, cert: &mut (dyn io::Read + Send + Sync))
                  -> Result<Box<dyn Encrypt<'a> + 'a>>;
 
+    /// Encrypts with the given certs.
+    ///
+    /// Like [`Encrypt::with_cert`], but for multiple certs.
+    fn with_certs(self: Box<Self>, certs: &mut (dyn io::Read + Send + Sync))
+                  -> Result<Box<dyn Encrypt<'a> + 'a>>;
+
     /// Encrypts the given data yielding the ciphertext.
     fn plaintext(self: Box<Self>,
                  plaintext: &'a mut (dyn io::Read + Send + Sync))
@@ -247,12 +271,19 @@ pub trait Decrypt<'a> {
                         cert: &mut (dyn io::Read + Send + Sync))
                         -> Result<Box<dyn Decrypt<'a> + 'a>>;
 
+    /// Adds the verification certs.
+    ///
+    /// Like [`Decrypt::verify_with_cert`], but for multiple certs.
+    fn verify_with_certs(self: Box<Self>,
+                         certs: &mut (dyn io::Read + Send + Sync))
+                         -> Result<Box<dyn Decrypt<'a> + 'a>>;
+
     /// Tries to decrypt with the given session key.
     fn with_session_key(self: Box<Self>, sk: SessionKey)
                         -> Result<Box<dyn Decrypt<'a> + 'a>>;
 
     /// Tries to decrypt with the given password.
-    fn with_password(self: Box<Self>, password: &str)
+    fn with_password(self: Box<Self>, password: Password)
                      -> Result<Box<dyn Decrypt<'a> + 'a>>;
 
     /// Adds the decryption key.
@@ -284,14 +315,52 @@ pub trait Dearmor<'a> {
         -> Result<Box<dyn Ready + 'a>>;
 }
 
-/// Normalizes the given password.
+/// A normalized password.
 ///
 /// See [Passwords are Human-Readable] in the SOP spec.
 ///
 ///   [Passwords are Human-Readable]: https://datatracker.ietf.org/doc/html/draft-dkg-openpgp-stateless-cli-02#section-7.8
-pub fn normalize_password<P: AsRef<str>>(p: P) -> String {
-    // XXX: Maybe do additional checks.
-    p.as_ref().trim_end().into()
+pub struct Password(Box<[u8]>);
+
+impl From<&str> for Password {
+    fn from(p: &str) -> Self {
+        p.to_string().into()
+    }
+}
+
+impl From<String> for Password {
+    fn from(p: String) -> Self {
+        Password(Self::normalize(p).into_bytes().into_boxed_slice())
+    }
+}
+
+impl AsRef<[u8]> for Password {
+    fn as_ref(&self) -> &[u8] {
+        &self.0
+    }
+}
+
+impl Password {
+    fn normalize(p: String) -> String {
+        // XXX: Maybe do additional checks.
+        let normalized = p.trim_end().to_string();
+
+        // Wipe original string.
+        let mut p = p.into_bytes();
+        unsafe {
+            memsec::memzero(p.as_mut_ptr(), p.len());
+        }
+
+        normalized
+    }
+}
+
+impl Drop for Password {
+    fn drop(&mut self) {
+        unsafe {
+            memsec::memzero(self.0.as_mut_ptr(), self.0.len());
+        }
+    }
 }
 
 /// An operation ready to be executed.
@@ -330,7 +399,7 @@ pub trait ReadyWithResult<T> {
     }
 }
 
-/// Represents a session key.
+/// A session key.
 pub struct SessionKey {
     algorithm: u8,
     key: Box<[u8]>,
@@ -370,6 +439,14 @@ impl fmt::Display for SessionKey {
     }
 }
 
+impl Drop for SessionKey {
+    fn drop(&mut self) {
+        unsafe {
+            memsec::memzero(self.key.as_mut_ptr(), self.key.len());
+        }
+    }
+}
+
 impl std::str::FromStr for SessionKey {
     type Err = ParseError;
     fn from_str(sk: &str) -> ParseResult<Self> {
